<?php

namespace Drupal\hierarchical_taxonomy_export\Form;

use Drupal\Core\Database\Database;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\taxonomy\Entity\Term;
use Drupal\taxonomy\Entity\Vocabulary;
use Symfony\Component\HttpFoundation\ResponseHeaderBag;

/**
 * HierarchicalTaxonomyExportForm class.
 */
class HierarchicalTaxonomyExportForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'hte_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $vocabulary_ids = array_keys(taxonomy_vocabulary_get_names());
    $vocabulary_list = array();
    foreach($vocabulary_ids as $vocabulary_id){
      $vocabulary = Vocabulary::Load($vocabulary_id);
      //rray_push($vocabulary_list,$vocabulary->label());
      $vocabulary_list[$vocabulary_id] = $vocabulary->label();
    }
    $form['vocabulary'] = [
      '#type' => 'select',
      '#title' => $this->t('Vocabulary'),
      '#options' => $vocabulary_list,
    ];

    $form['term_depth'] = [
      '#type' => 'select',
      '#title' => $this->t('Term Depth'),
      '#options' => array(1,2,3),
    ];

    $form['actions'] = [
      '#type' => 'actions',
    ];

    // Add a submit button that handles the submission of the form.
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Export as CSV'),
    ];

    return $form;

  }

  /**
   * Form submission handler.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
      
      $vocabulary = $form_state->getValue('vocabulary');
      $depth = $form_state->getValue('term_depth');
      $connection = Database::getConnection();
      $count = -1;
      $array = array();
      $level1 = $connection->query("select entity_id from {taxonomy_term__parent} where parent_target_id=0 and bundle=:vocabulary",[':vocabulary' => $vocabulary]);
      if($level1) {
        while($row = $level1->fetchAssoc()){
          $count++;
          $term = Term::load($row['entity_id']);
          $category = $term->getName();
          $array[$count][0] = $category;
          $array[$count][1] = null;
          $array[$count][2] = null;
          if($depth > 0) {
            $level2 = $connection->query("select entity_id from {taxonomy_term__parent} where bundle=:vocabulary and parent_target_id=:id",[':vocabulary' => $vocabulary, ':id' => $row['entity_id']]);
            if($level2){
              while($row = $level2->fetchAssoc()){
                $count++;
                $term = Term::load($row['entity_id']);
                $category = $term->getName();
                $array[$count][0] = NULL;
                $array[$count][1] = $category;
                $array[$count][2] = NULL;
                if($depth > 1) {
                  $level3 = $connection->query("select entity_id from {taxonomy_term__parent} where bundle=:vocabulary and parent_target_id=:id",[':vocabulary' => $vocabulary, ':id' => $row['entity_id']]);
                  if($level3){
                    while($row = $level3->fetchAssoc()){
                      $count++;
                      $term = \Drupal\taxonomy\Entity\Term::load($row['entity_id']);
                      $category = $term->getName();
                      $array[$count][0] = NULL;
                      $array[$count][1] = NULL;
                      $array[$count][2] = $category;
                    }
                  }
                }
              }
            }
          }
        }       
      }

      header("Content-Disposition: attachment; filename=\"demo.xls\"");
      header("Content-Type: application/vnd.ms-excel;");
      header("Pragma: no-cache");
      header("Expires: 0");
      $dir = 'public://hte';
      if (file_prepare_directory($dir, FILE_CREATE_DIRECTORY)) {
        $csv_filename = $vocabulary;
        $csv_filepath = $dir . '/' . $csv_filename.".csv";
        $targs = [
          ':csv_url' => file_create_url($csv_filepath),
          '@csv_filename' => $csv_filename,
          '@csv_filepath' => $csv_filepath,
        ];
        if ($handle = fopen($csv_filepath, 'w+')) {
          foreach ($array as $data) {
            fputcsv($handle, $data,",");
          }
          fclose($handle);
        }
        drupal_set_message(t('Vocabulary Export Complete. You may download the CSV here: <a href=":csv_url">@csv_filename</a>', $targs), 'status');
      }
    }

}
